@@ -1,16 +1,19 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2010-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2010-2014 -- leonerd@leonerd.org.uk
 
 package IO::Socket::IP;
+# $VERSION needs to be set before  use base 'IO::Socket'
+#  - https://rt.cpan.org/Ticket/Display.html?id=92107
+BEGIN {
+   $VERSION = '0.31_001';
+}
 
 use strict;
 use warnings;
 use base qw( IO::Socket );
 
-our $VERSION = '0.21_001';
-
 use Carp;
 
 use Socket 1.97 qw(
@@ -22,16 +25,19 @@ use Socket 1.97 qw(
    IPPROTO_IPV6 IPV6_V6ONLY
    NI_DGRAM NI_NUMERICHOST NI_NUMERICSERV NIx_NOHOST NIx_NOSERV
    SO_REUSEADDR SO_REUSEPORT SO_BROADCAST SO_ERROR
-   SOCK_DGRAM SOCK_STREAM 
+   SOCK_DGRAM SOCK_STREAM
    SOL_SOCKET
 );
 my $AF_INET6 = eval { Socket::AF_INET6() }; # may not be defined
 my $AI_ADDRCONFIG = eval { Socket::AI_ADDRCONFIG() } || 0;
 use POSIX qw( dup2 );
-use Errno qw( EINVAL EINPROGRESS EISCONN );
+use Errno qw( EINVAL EINPROGRESS EISCONN ETIMEDOUT EWOULDBLOCK );
 
 use constant HAVE_MSWIN32 => ( $^O eq "MSWin32" );
 
+# At least one OS (Android) is known not to have getprotobyname()
+use constant HAVE_GETPROTOBYNAME => defined eval { getprotobyname( "tcp" ) };
+
 my $IPv6_re = do {
    # translation of RFC 3986 3.2.2 ABNF to re
    my $IPv4address = do {
@@ -58,8 +64,7 @@ my $IPv6_re = do {
 
 =head1 NAME
 
-C<IO::Socket::IP> - A drop-in replacement for C<IO::Socket::INET> supporting
-both IPv4 and IPv6
+C<IO::Socket::IP> - Family-neutral IP socket supporting both IPv4 and IPv6
 
 =head1 SYNOPSIS
 
@@ -80,7 +85,7 @@ both IPv4 and IPv6
 =head1 DESCRIPTION
 
 This module provides a protocol-independent way to use IPv4 and IPv6 sockets,
-as a drop-in replacement for L<IO::Socket::INET>. Most constructor arguments
+intended as a replacement for L<IO::Socket::INET>. Most constructor arguments
 and methods are provided in a backward-compatible way. For a list of known
 differences, see the C<IO::Socket::INET> INCOMPATIBILITES section below.
 
@@ -91,7 +96,7 @@ falling back to IPv4-only on systems which don't.
 
 =head1 REPLACING C<IO::Socket> DEFAULT BEHAVIOUR
 
-By placing C<-register> in the import list, C<IO::Socket> uses
+By placing C<-register> in the import list, L<IO::Socket> uses
 C<IO::Socket::IP> rather than C<IO::Socket::INET> as the class that handles
 C<PF_INET>.  C<IO::Socket> will also use C<IO::Socket::IP> rather than
 C<IO::Socket::INET6> to handle C<PF_INET6>, provided that the C<AF_INET6>
@@ -286,15 +291,10 @@ If your platform does not support disabling this option but you still want to
 listen for both C<AF_INET> and C<AF_INET6> connections you will have to create
 two listening sockets, one bound to each protocol.
 
-=item Timeout
-
-This C<IO::Socket::INET>-style argument is not currently supported. See the
-C<IO::Socket::INET> INCOMPATIBILITES section below.
-
 =item MultiHomed
 
 This C<IO::Socket::INET>-style argument is ignored, except if it is defined
-but false. See the C<IO::Socket::INET> INCOMPATIBILITES section below. 
+but false. See the C<IO::Socket::INET> INCOMPATIBILITES section below.
 
 However, the behaviour it enables is always performed by C<IO::Socket::IP>.
 
@@ -304,6 +304,18 @@ If defined but false, the socket will be set to non-blocking mode. Otherwise
 it will default to blocking mode. See the NON-BLOCKING section below for more
 detail.
 
+=item Timeout => NUM
+
+If defined, gives a timeout in seconds to wait per C<connect()> call. Note
+that if the hostname resolves to multiple address candidates, the same timeout
+will apply to each connection attempt individually, rather than to the
+operation as a whole. Further note that the timeout does not apply to the
+initial hostname resolve operation, if connecting by hostname.
+
+This behviour is copied directly form C<IO::Socket::INET>; for more fine-
+grained control over connection timeouts, consider performing a nonblocking
+connect directly.
+
 =back
 
 If neither C<Type> nor C<Proto> hints are provided, a default of
@@ -311,6 +323,12 @@ C<SOCK_STREAM> and C<IPPROTO_TCP> respectively will be set, to maintain
 compatibility with C<IO::Socket::INET>. Other named arguments that are not
 recognised are ignored.
 
+If neither C<Family> nor any hosts or addresses are passed, nor any
+C<*AddrInfo>, then the constructor has no information on which to decide a
+socket family to create. In this case, it performs a C<getaddinfo> call with
+the C<AI_ADDRCONFIG> flag, no host name, and a service name of C<"0">, and
+uses the family of the first returned result.
+
 If the constructor fails, it will set C<$@> to an appropriate error message;
 this may be from C<$!> or it may be some other string; not every failure
 necessarily has an associated C<errno> value.
@@ -324,7 +342,7 @@ behaviour given in the C<PeerHost> AND C<LocalHost> PARSING section below.
 
 =cut
 
-sub new 
+sub new
 {
    my $class = shift;
    my %arg = (@_ == 1) ? (PeerHost => $_[0]) : @_;
@@ -354,17 +372,18 @@ sub configure
       my $host    = $type . 'Host';
       my $service = $type . 'Service';
 
-      if (exists $arg->{$host} && !exists $arg->{$service}) {
-         defined $arg->{$host} or next;
+      if( defined $arg->{$host} ) {
          ( $arg->{$host}, my $s ) = $self->split_addr( $arg->{$host} );
+         # IO::Socket::INET compat - *Host parsed port always takes precedence
          $arg->{$service} = $s if defined $s;
       }
    }
 
-   $self->_configure( $arg );
+   $self->_io_socket_ip__configure( $arg );
 }
 
-sub _configure
+# Avoid simply calling it _configure, as some subclasses of IO::Socket::INET on CPAN already take that
+sub _io_socket_ip__configure
 {
    my $self = shift;
    my ( $arg ) = @_;
@@ -390,7 +409,9 @@ sub _configure
 
    if( defined( my $proto = $arg->{Proto} ) ) {
       unless( $proto =~ m/^\d+$/ ) {
-         my $protonum = getprotobyname( $proto );
+         my $protonum = HAVE_GETPROTOBYNAME
+            ? getprotobyname( $proto )
+            : eval { Socket->${\"IPPROTO_\U$proto"}() };
          defined $protonum or croak "Unrecognised protocol $proto";
          $proto = $protonum;
       }
@@ -512,13 +533,29 @@ sub _configure
       }
    }
 
-   if( !@infos and defined $hints{family} ) {
+   if( !@infos ) {
       # If there was a Family hint then create a plain unbound, unconnected socket
-      @infos = ( {
-         family   => $hints{family},
-         socktype => $hints{socktype},
-         protocol => $hints{protocol},
-      } );
+      if( defined $hints{family} ) {
+         @infos = ( {
+            family   => $hints{family},
+            socktype => $hints{socktype},
+            protocol => $hints{protocol},
+         } );
+      }
+      # If there wasn't, use getaddrinfo()'s AI_ADDRCONFIG side-effect to guess a
+      # suitable family first.
+      else {
+         ( my $err, @infos ) = getaddrinfo( "", "0", \%hints );
+         if( $err ) {
+            $@ = "$err";
+            $! = EINVAL;
+            return;
+         }
+
+         # We'll take all the @infos anyway, because some OSes (HPUX) are known to
+         # ignore the AI_ADDRCONFIG hint and return AF_INET6 even if they don't
+         # support them
+      }
    }
 
    # In the nonblocking case, caller will be calling ->setup multiple times.
@@ -586,6 +623,13 @@ sub setup
             return 0;
          }
 
+         # If connect failed but we have no system error there must be an error
+         # at the application layer, like a bad certificate with
+         # IO::Socket::SSL.
+         # In this case don't continue IP based multi-homing because the problem
+         # cannot be solved at the IP layer.
+         return 0 if ! $!;
+
          ${*$self}{io_socket_ip_errors}[0] = $!;
          next;
       }
@@ -609,7 +653,45 @@ sub connect
    # useful APIs I'm just going to end-run around it and call CORE::connect()
    # directly
 
-   return CORE::connect( $self, $_[0] ) if @_;
+   if( @_ ) {
+      my ( $addr ) = @_;
+
+      # Annoyingly IO::Socket's connect() is where the timeout logic is
+      # implemented, so we'll have to reinvent it here
+      my $timeout = ${*$self}{'io_socket_timeout'};
+
+      return CORE::connect( $self, $addr ) unless defined $timeout;
+
+      my $was_blocking = $self->blocking( 0 );
+
+      my $err = defined CORE::connect( $self, $addr ) ? 0 : $!+0;
+
+      if( !$err ) {
+         # All happy
+         return 1;
+      }
+      elsif( not( $err == EINPROGRESS or $err == EWOULDBLOCK ) ) {
+         # Failed for some other reason
+         return undef;
+      }
+      elsif( !$was_blocking ) {
+         # We shouldn't block anyway
+         return undef;
+      }
+
+      my $vec = ''; vec( $vec, $self->fileno, 1 ) = 1;
+      if( !select( $vec, $vec, $vec, $timeout ) ) {
+         $! = ETIMEDOUT;
+         return undef;
+      }
+
+      # Hoist the error by connect()ing a second time
+      $err = defined CORE::connect( $self, $addr ) ? 0 : $!+0;
+      $self->blocking( $was_blocking );
+
+      $! = $err, return undef if $err;
+      return 1;
+   }
 
    return 1 if !${*$self}{io_socket_ip_connect_in_progress};
 
@@ -626,7 +708,7 @@ sub connect
    # (still in progress). This even works on MSWin32.
    my $addr = ${*$self}{io_socket_ip_infos}[${*$self}{io_socket_ip_idx}]{peeraddr};
 
-   if( $self->connect( $addr ) or $! == EISCONN ) {
+   if( CORE::connect( $self, $addr ) or $! == EISCONN ) {
       delete ${*$self}{io_socket_ip_connect_in_progress};
       $! = 0;
       return 1;
@@ -928,8 +1010,7 @@ To support the C<IO::Socket::INET> API, the host and port information may be
 passed in a single string rather than as two separate arguments.
 
 If either C<LocalHost> or C<PeerHost> (or their C<...Addr> synonyms) have any
-of the following special forms, and C<LocalService> or C<PeerService> (or
-their C<...Port> synonyms) are absent, special parsing is applied.
+of the following special forms then special parsing is applied.
 
 The value of the C<...Host> argument will be split to give both the hostname
 and port (or service name):
@@ -950,6 +1031,10 @@ service name and number, in a form such as
 In this case, the name (C<http>) will be tried first, but if the resolver does
 not understand it then the port number (C<80>) will be used instead.
 
+If the C<...Host> argument is in this special form and the corresponding
+C<...Service> or C<...Port> argument is also defined, the one parsed from
+the C<...Host> argument will take precedence and the other will be ignored.
+
 =head2 ( $host, $port ) = IO::Socket::IP->split_addr( $addr )
 
 Utility method that provides the parsing functionality described above.
@@ -1048,8 +1133,6 @@ sub configure
 
 =item *
 
-The C<Timeout> constructor argument is currently not recognised.
-
 The behaviour enabled by C<MultiHomed> is in fact implemented by
 C<IO::Socket::IP> as it is required to correctly support searching for a
 useable address from the results of the C<getaddrinfo(3)> call. The
